;======================================================================
;
; Author        : Andreas Reber
;
; Date          : March 2005
;
; File          : USB_I2C.asm
;
; Hardware      : ADuC832 8,3886 MHz   1,43 s / MA   
;
; Description   : Software for USB to I2C Dongle 
;		 
;               
;======================================================================

$include(MOD832.h)		; Symbols ADuC832

$SET (DEBUG = 0)

pDebug		DATA	2FH	; 
BITCNT          DATA    30H     ; bit counter for I2C routines
pSLAVEADD       DATA    31H     ; slave address for I2C routines
pSend		DATA	32H
pRec		DATA	33H
pCRC		DATA	34H
pCmd_Return	DATA	35H	; answer from I2C Cmd
pTimeOut	DATA	36H	; Counter for TimeOut
pDataLength	DATA	37H	; 
pCmd		DATA	38H	; Frame Command
pBytesSend	DATA	39H	; bytes send by I2C before error
pErasePages	DATA	3AH	; pages to erase
pEF_Pages	DATA	44H	


NOACK           BIT     00H     ; I2C no acknowledge flag
bnACK		BIT	01H	; no ACK
bMoreBytes	BIT	02H	; read more than one byte from slave
bTimeOut	BIT	03H	; TimeOut valid
bFrame		BIT	04H	; Debug Flag for data in pDebug


pUSB		EQU	8000H	; virtual address for USB Chip		
bTXE		BIT	P1.0	; /TXE USB Transmit emty
bRXF		BIT	P1.1	; /RXF USB Receive full
bUSB_OK		BIT	P1.2	; /PWREN USB ready for Transmit


bT1		BIT	P1.5
bT2		BIT	P1.6
bT3		BIT	P1.7
bLED1		BIT	P3.2
bLED2		BIT	P3.3
bLED3		BIT	P3.4

iTimeOut	EQU	100d	; TimeOut after 200 ms * iTimeOut
pEraseBuffer	EQU	40h	; begin data for erase


jmp _START


ORG 0100h
; *******************************************************************************
; *	Startup
; *******************************************************************************
_START: anl	PLLCON,#41h	; PLL on 8.3886 MHz
	mov	SP,#7Fh 	; Stack 
	
        mov	T3CON,#81h	; RS232 115k2  83 for 16,7772 and 57k6
	mov	T3FD,#09h
	mov	SCON,#52h


; initialise settings
	mov     pSLAVEADD,#04h  ; setting slave address
	CALL	_MakeSlave	; Slave mode until command from PC 
	
	mov	P2,#80H		; Dummy for USB
	mov	R0,#00H

	mov	40h,#07h	; Start ID
	mov	41h,#0Eh	; Start ID

	clr	bTXE
	clr	bRXF
	clr	bUSB_OK

	mov	20h,#0h		; clear flags

	jb	bUSB_OK,$	; waiting USB ready

; *******************************************************************************
; *	Main
; *******************************************************************************
_MAIN:	jb	bRXF,$		; waiting flag low 	
	movx	A,@R0		; read byte
					
	cjne	A,#02h,_ARM	; expected 0x02h for dongle identification

	CALL	_HEX2_COM	; Debug
	CALL	_RET_COM

	mov	A,#06h		; I2C-ADuC ok
	jb	bTXE,$		; waiting flag low 	
	movx	@R0,A		; write byte

	CALL	_HEX2_COM	; Debug
	CALL	_RET_COM
	
	CALL	_USB_Clear	; garbagge in RX -> clear
       	jmp	_MAIN
					
_ARM:	cjne	A,#08h,_CMD	; expected 0x08h for ADuC-Ini

	CALL	_HEX2_COM	; Debug
	CALL	_RET_COM

	mov	pRec,#24d	; 24 byte ID-String

	CALL	_MakeMaster
	CALL	_I2C_ID		;
	CALL	_MakeSlave

	jmp	_MAIN		; back for next command 
	
_CMD: 	cjne	A,#07h,_DIC	; expected 0x07h for command
	CALL	_USB_READ_BYTE	; read second byte 
	cjne	A,#0Eh,_W_CMD	; expected 0x0E for commandframe
	CALL	_USB_READ_BYTE	; read length 
	mov	pDataLength,A
	mov	pSend,A		; number of bytes for I2C to send		
	CALL	_USB_READ_BYTE	; read command
	mov	pCmd,A		; store command
		
	jmp	_PARSER


_DIC:	cjne	A,#03h,_W_CMD	; Dongle-Ident frame
	jb	bRXF,$		; waitung flag low 	
	movx	A,@R0		; read byte

	cjne	A,#01h,_UIDC
	mov	DPTR,#DongId	; 
	CALL	_STRING_USB	; send Frame
	CALL	_USB_Clear	; clear RX
	jmp	_MAIN

_UIDC:	cjne	A,#02h,_W_CMD
	mov	DPTR,#UserId	; 
	CALL	_STRING_USB	; send Frame
        CALL	_USB_Clear	; clear RX
	jmp	_MAIN

	
			
_W_CMD: ;mov	A,#07h		; wrong command send NAK
	;jb	bTXE,$		; waiting flag low 	
	;movx	@R0,A		; write byte

	CALL	_USB_Clear	; clear RX

	jmp	_MAIN
		

_PARSER:
	cjne	A,#'E',_W	; Erase Page
	
	mov	R1,#pEraseBuffer
	mov	R3,#6d
_ECRL:	jb	bRXF,$		; flag high -> retry 	
	movx	A,@R0		; read byte from USB
	mov	@R1,A		; store
	inc	R1
	djnz	R3,_ECRL
		
	mov	A,pEF_Pages

	jnz	_NME
	mov	A,#124d		; Pages for MassErase
_NME:	mov	pErasePages,A	; store

$IF (DEBUG = 1)
	CALL	_HEX2_COM	; Debug
	mov	A,#'p'		
	CALL	SENDCHAR
	CALL	_RET_COM
$ENDIF


	CALL	_MakeMaster
	CALL	_I2C
	CALL	_MakeSlave

	CALL	_USB_Clear	; clear RX

$IF (DEBUG = 1)
	CALL	_RET_COM
$ENDIF

	mov	A,pCmd_Return
	jb	bTXE,$		; waiting flag low 	
	movx	@R0,A		; write byte

$IF (DEBUG = 1)
	CALL	_HEX2_COM	; Debug
	mov	A,#'e'		
	CALL	SENDCHAR
	CALL	_RET_COM
$ENDIF
	
	jmp	_MAIN

_W:	cjne	A,#'W',_V	; Write

	CALL	_MakeMaster
	CALL	_I2C
	CALL	_MakeSlave

	CALL	_USB_Clear	; clear RX

$IF (DEBUG = 1)
	CALL	_RET_COM
$ENDIF
	
	mov	A,pCmd_Return
	jb	bTXE,$		; waiting flag low 	
	movx	@R0,A		; write byte

$IF (DEBUG = 1)
	CALL	_HEX2_COM	; Debug
	mov	A,#'w'		
	CALL	SENDCHAR
	CALL	_RET_COM
$ENDIF

	jmp	_MAIN

_V:	cjne	A,#'V',_R	; Verify

	CALL	_MakeMaster
	CALL	_I2C
	CALL	_MakeSlave

	CALL	_USB_Clear	; clear RX

$IF (DEBUG = 1)
	CALL	_RET_COM
$ENDIF

	mov	A,pCmd_Return
	jb	bTXE,$		; waiting flag low 	
	movx	@R0,A		; write byte

$IF (DEBUG = 1)
	CALL	_HEX2_COM	; Debug
	mov	A,#'v'		
	CALL	SENDCHAR
	CALL	_RET_COM
$ENDIF
	
	jmp	_MAIN

_R:	cjne	A,#'R',_W_CMD	; Run

	CALL	_MakeMaster	
	CALL	_I2C
	CALL	_MakeSlave

	CALL	_USB_Clear	; clear RX

$IF (DEBUG = 1)
	CALL	_RET_COM
$ENDIF

	mov	A,pCmd_Return
	jb	bTXE,$		; waiting flag low 	
	movx	@R0,A		; write byte

$IF (DEBUG = 1)
	CALL	_HEX2_COM	; Debug
	mov	A,#'r'		
	CALL	SENDCHAR
	CALL	_RET_COM
$ENDIF

	jmp	_MAIN

			
; *******************************************************************************
; *	Transfer over I2C
; *	pSend :	Bytes to send
; *	pCmd_Return : BEL or ACK as answer 
; *******************************************************************************
_I2C:	
;	mov	pCmd_Return,#06h	; TEST
;	ret				; TEST

	push	PSW
	clr	NOACK
;	mov	R1,#pDebug		; Debug
	clr	bFrame
	clr	bTimeOut	

	CALL    STARTBIT        	; acquire bus and send slave address

        mov     A, pSLAVEADD
        CALL    SENDBYTE        	; sets NOACK if NACK received
        jnb      NOACK, _SB		; if  acknowledge send bytes
$IF (DEBUG = 1)
	CALL	_RET_COM
	mov	DPTR,#DT1
	CALL	_STRING_COM
	CALL	_RET_COM	
$ENDIF
	jmp	_STOPP_T	

        ; sending bytes 
_SB:	mov	pBytesSend,#0h		; clear send byte counter
	mov	A,#07h

$IF (DEBUG = 1)
	mov	pDebug,#07h
	CALL	_HEX2_COM
$ENDIF

	CALL    SENDBYTE		; send byte via I2C	
	jb      NOACK, _STOPP_T		; if no acknowledge send stop


	mov	A,#0Eh

$IF (DEBUG = 1)	
	mov	pDebug,#0Eh
	CALL	_HEX2_COM
$ENDIF

	CALL    SENDBYTE		; send byte via I2C
	jb      NOACK, _STOPP_T		; if no acknowledge send stop

	mov	A,pDataLength

$IF (DEBUG = 1)
	mov	pDebug,pDataLength
	CALL	_HEX2_COM
$ENDIF
	CALL    SENDBYTE		; send byte via I2C
 
;	setb	bFrame			; data in Debug FiFo
	jb      NOACK, _STOPP_T		; if no acknowledge send stop

	mov	A,pCmd

$IF (DEBUG = 1)
	mov	pDebug,pCmd
	CALL	_HEX2_COM
$ENDIF

	CALL    SENDBYTE		; send byte via I2C 

	jb      NOACK, _STOPP_T		; if no acknowledge send stop

	cjne	A,#'E',_NO_ERASE
	
	mov	R1,#pEraseBuffer
	mov	R3,#6d
_ECSL:	mov	A,@R1			; read byte from buffer

$IF (DEBUG = 1)
	mov	pDebug,A
	CALL	_HEX2_COM
$ENDIF

	CALL    SENDBYTE		; send byte via I2C
	jb      NOACK, _STOPP_T		; if no acknowledge send stop
	inc	R1
	djnz	R3,_ECSL
	
	mov	R7,pErasePages		; pages to erase
_E_WAIT:
	CALL	_21ms
	djnz	R7,_E_WAIT

	jmp	_SRRA			; send ready read answer	



; waiting for data from USB with TimeOut (1000 MCs)
_NO_ERASE:
	mov	R2,pSend

	mov	R5,#250d
_TL:	jb	bRXF,_TOU1		; flag high -> no data 	
	movx	A,@R0			; read byte from USB
	jmp	_SBF1
 
_TOU1:	;CALL	_10ms
	djnz	R5,_TL			; dec TimeOut counter
	setb	bTimeOut		; debug

$IF (DEBUG = 1)
	CALL	_RET_COM		; debug
	mov	DPTR,#DT9
	CALL	_STRING_COM
	CALL	_RET_COM	
$ENDIF

	jmp	_STOPP_T		

_SBF1:	CALL    SENDBYTE		; send byte via I2C

$IF (DEBUG = 1)
	CALL	_HEX2_COM
;	CALL	_RET_COM	
$ENDIF

	mov	R5,#250d		; reload, because 250d for each byte
	jb      NOACK, _STOPP_T		; if no acknowledge send stop
	djnz	R2,_TL	


	; frame ready
_SRRA:	CALL	STOPBIT			; message send

	CALL	_Delay_K       		; wait, because Slave must work

; sequenz read answer	
	CALL	STARTBIT

	mov     A, pSLAVEADD
	inc	A			; read I2C
        CALL    SENDBYTE        	; send address

$IF (DEBUG = 1)
	mov	pDebug,#'r'	
$ENDIF

	jb      NOACK, _STOPP_T		; if no acknowledge send stop
	CALL    RCVBYTE_N       	; receive data byte
	mov	pCmd_Return,A
	
_STOPP_T:  
        CALL    STOPBIT         	; sends stop bit

	jnb	bTimeOut,_NOOC

$IF (DEBUG = 1)
	CALL	_RET_COM		; debug
	mov	A,#'T'
	CALL	SENDCHAR
;	mov	DPTR,#DT15
;	CALL	_STRING_COM
;	mov	A,pDebug
;	CALL	_HEX2_COM
	CALL	_RET_COM
$ENDIF

	mov	pCmd_Return,#08h

	clr	bFrame

_NOOC:	jnb     NOACK,_FinT

$IF (DEBUG = 1)
	CALL	_RET_COM		; debug
;	mov	DPTR,#DT16
;	CALL	_STRING_COM
;	mov	A,pDebug
;	CALL	_HEX2_COM
	mov	A,#'N'
	CALL	SENDCHAR
	CALL	_RET_COM
$ENDIF

	mov	pCmd_Return,#07h
	pop	PSW
	ret

_FinT:
_NRB:	pop	PSW	
	ret

; *******************************************************************************
; *	Send 0x08, waiting for ACK and then read the ID-String
; *******************************************************************************
_I2C_ID:	
	push	02h			; R2	
	push	PSW
	clr	NOACK
	mov	pTimeOut,#iTimeOut	; Timeout


; waits in loop for ACK for the slave address (if slave ok, he would ack his address)
; loop will terminated by a TimeOut (20 s)
_INIL:	
	CALL    STARTBIT        	; acquire bus and send slave address
 	mov     A, pSLAVEADD
        CALL    SENDBYTE        	; sets NOACK if NACK received

        jnb     NOACK, _NB1		; if acknowledge send 0x08

	CALL	STOPBIT	

	CALL	_200ms

	djnz	pTimeOut,_INIL		; break after TimeOut

	mov	A,#08h			; TimeOut value
	jb	bTXE,$			; waitung flag low 	
	movx	@R0,A			; write byte to USB
	mov	pCmd_Return,A

$IF (DEBUG = 1)
	mov	DPTR,#DT6		; Debug
	CALL	_STRING_COM
	CALL	_RET_COM
$ENDIF
	jmp	_STOPPD


        ; sending 0x08
_NB1:	
	mov	A,#08h
        CALL    SENDBYTE        	; sets NOACK if NACK received

	jnb     NOACK,_NB2		; if no acknowledge send stop
$IF (DEBUG = 1)
	mov	DPTR,#DT7
	CALL	_STRING_COM
	CALL	_RET_COM
$ENDIF
	jmp	_STOPPD
	
	; frame ready
_NB2:	CALL	STOPBIT			; end write sequence

	CALL	_10ms

; sequenz read answer	
	CALL	STARTBIT		; begin read sequence

	mov     A, pSLAVEADD
	inc	A			; read from I2C
        CALL    SENDBYTE        	; send address

	jnb      NOACK,_RLD		; if no acknowledge send stop
$IF (DEBUG = 1)
	mov	DPTR,#DT8
	CALL	_STRING_COM
	CALL	_RET_COM

	jmp	_STOPPD
$ENDIF

; read 24 bytes from slave	
	mov	R2,#23d		
_RLD:	CALL    RCVBYTE_A       	; Receive data byte with ACK for slave.
	jb	bTXE,$			; waitung flag low 	
	movx	@R0,A			; write byte to USB
	CALL	SENDCHAR
	djnz	R2,_RLD


	CALL    RCVBYTE_N       	; Receive data byte without ACK for slave.
	jb	bTXE,$			; waitung flag low 	
	movx	@R0,A			; write byte to USB

	CALL	SENDCHAR
	CALL	_RET_COM


_STOPPD:  
        CALL    STOPBIT         	; sends stop bit
	jnb      NOACK,_FinTD
	mov	A,#07h			; NAK
       	jb	bTXE,$			; waitung flag low 	
	movx	@R0,A			; write byte to USB
	mov	pCmd_Return,A
	pop	PSW
	pop	02h			; R2
	ret

_FinTD:
	pop	PSW
	pop	02h			; R2
	ret	
	
; *******************************************************************************
; *	Read a byte from USB
; *  	Byte in A
; *******************************************************************************
_USB_READ_BYTE:
	jb	bRXF,$			; waitung flag low 	
	movx	A,@R0			; read byte	
	ret

; *******************************************************************************
; *	Write a byte to USB
; *  	Byte in A
; *******************************************************************************
_USB_WRITE_BYTE:
	jb	bTXE,$			; waitung flag low 	
	movx	@R0,A			; write byte	
	ret

; *******************************************************************************
; *	Writes strings to USB
; * 	Pointer to string in DPTR
; *******************************************************************************
_STRING_USB:	
	clr	A		 
	movc	A,@A+DPTR		; get character
	jz	_FINU			; string end
	jb	bTXE,$			; waitung flag low
	movx	@R0,A			; char -> USB
	inc	DPTR			; pointer ++
	jmp	_STRING_USB
_FINU:	
	ret

; *******************************************************************************
; *	Writes strings to COM
; * 	Pointer to string in DPTR
; *******************************************************************************
_STRING_COM:
	push	ACC	
	clr	A		 
	movc	A,@A+DPTR		; get character
	jz	_FINC			; string end ?
	jnb     TI,$            	; wait til present char gone
        clr     TI              	; must clear TI
        mov     SBUF,A			; char to COM
	inc	DPTR			; pointer ++
	jmp	_STRING_COM
_FINC:	
	pop	ACC
	ret

; *******************************************************************************
; *	Sends the start bit to initiate an I2C communication
; *******************************************************************************
STARTBIT:
        SETB    MDE             ; enable SDATA pin as an output
        CLR     NOACK
        CLR     MDO             ; low O/P on SDATA 
	nop
        CLR     MCO             ; start bit
        RET

; *******************************************************************************
; *	Sends the stop bit to end an I2C transmission
; *******************************************************************************
STOPBIT:
        SETB    MDE             ; to enable SDATA pin as an output
        CLR     MDO             ; get SDATA ready for stop
        SETB    MCO             ; set clock for stop
	nop
        SETB    MDO             ; this is the stop bit
        RET

; *******************************************************************************
; *	Send 8-bits in ACC to the slave
; *******************************************************************************
SENDBYTE:
	push	ACC
	clr	NOACK		
        MOV     BITCNT,#8       ; 8 bits in a byte
        SETB    MDE             ; to enable SDATA pin as an output
        CLR     MCO             ; make sure that the clock line is low
SENDBIT:
        RLC     A               ; put data bit to be sent into carry
        MOV     MDO,C           ; put data bit on SDATA line
        SETB    MCO             ; clock to send bit
	nop
	nop
        CLR     MCO             ; clear clock 
        DJNZ    BITCNT,SENDBIT  ; jump back and send all eight bits

        CLR     MDE             ; release data line for acknowledge
	nop
        SETB    MCO             ; send clock for acknowledge
        JNB     MDI,NEXT        ; this is a check for acknowledge
        SETB    NOACK           ; no acknowledge, set flag
NEXT:   CLR     MCO             ; clear clock 
	pop	ACC
        RET

; *******************************************************************************
; *	receives one byte of data from an I2C slave device without ACK. 
; *	Byte in A
; *******************************************************************************
RCVBYTE_N:
;        push	ACC
	MOV     BITCNT,#8       ; Set bit count.
        CLR     MDE             ; to enable SDATA pin as an input
	nop
        CLR     MCO             ; make sure the clock line is low
RCVBITN:  
        SETB    MCO             ; clock to recieve bit
	nop
        CLR     MCO             ; clear clock
        MOV     C,MDI           ; read data bit into carry.
        RLC     A               ; Rotate bit into result byte.

        DJNZ    BITCNT,RCVBITN  ; Repeat until all bits received.
                                ; recieved byte is in the accumulator

        SETB    MDE             ; Data pin =Output for NACK
        SETB    MDO             ; Send NACK (always send NACK for
                                ; last byte in transmission)
        SETB    MCO             ; Send NACK clock.
	nop
        CLR     MCO
	clr	MDE		; Data pin =Input
;	pop	ACC
        RET

; *******************************************************************************
; *	receives one byte of data from an I2C slave device with ACK. 
; *	Byte in A
; *******************************************************************************
RCVBYTE_A:
        MOV     BITCNT,#8       ; Set bit count.
        CLR     MDE             ; to enable SDATA pin as an input	
        CLR     MCO             ; make sure the clock line is low
RCVBITA:  
        SETB    MCO             ; clock to recieve bit

        CLR     MCO             ; clear clock
        MOV     C,MDI           ; read data bit into carry.
        RLC     A               ; Rotate bit into result byte.

        DJNZ    BITCNT,RCVBITA  ; Repeat until all bits received.
                                ; recieved byte is in the accumulator

        SETB    MDE             ; Data pin =Output for NACK
        clr    	MDO            	; Send NACK (always send NACK for
                                ; last byte in transmission)
        SETB    MCO             ; Send NACK clock.
	nop
        CLR     MCO
	clr	MDE		; Data pin =Input
        RET


; *******************************************************************************
; *	set the I2C Interface to Master 
; *******************************************************************************
_MakeMaster:
        mov     I2CCON,#0A8h    ; sets SDATA & SCLOCK, and
                                ; selects master mode
        CLR     NOACK
        clr     bnACK
	setb	MCO
	ret

; *******************************************************************************
; *	set the I2C Interface to Slave
; *******************************************************************************
_MakeSlave:	
        mov     I2CCON,#00h    ; sets slave mode
	ret

; *******************************************************************************
; *     Convert a byte to BCD and send it to USB
; *	Byte in A
; *******************************************************************************
_HEX2_USB:
        push    B
        push    PSW

        mov     B, #100d
        div     AB
        add     A, #30h
        jb	bTXE,$		; waitung flag low	
	movx	@R0,A		; write byte

        mov     A, B
        mov     B, #10d
        div     AB
        add     A, #30h
        jb	bTXE,$		; waitung flag low	
	movx	@R0,A		; write byte

        mov     A, B
        add     A, #30h
        jb	bTXE,$		; waitung flag low	
	movx	@R0,A		; write byte

        pop     PSW
        pop     B
        ret


; *******************************************************************************
; *     send command answer to PC
; *	Byte in A
; *******************************************************************************
_RET_COM:
	push	ACC

	mov	A,#13		; CR
	CALL	SENDCHAR
	mov	A,#10		; LF
	CALL	SENDCHAR

	pop	ACC
	ret

; *******************************************************************************
; *     Convert a byte to 2 chars and send it to COM
; *	Byte in A
; *******************************************************************************
_HEX2_COM:
        push    ACC
        push    PSW

        mov     B, A
	swap 	A
        anl	A,#0Fh
	CALL	HEX2ASCII
	CALL	SENDCHAR

        mov     A, B
        anl	A,#0Fh
	CALL	HEX2ASCII
	CALL	SENDCHAR

	mov	A,#20h
	CALL	SENDCHAR

        pop     PSW
        pop     ACC
        ret

; *******************************************************************************
; *   Clear data in USB RX 
; *******************************************************************************
_USB_Clear:
	push	ACC
_CLL:	jb	bRXF,_FCL	; if no data in USB RX, return 	
	movx	A,@R0		; read

$IF (DEBUG = 1)
;	CALL 	_HEX2_COM	
$ENDIF
	
	jmp	_CLL		; loop to clear RX buffer

_FCL:	pop	ACC
	ret			



; *******************************************************************************
; *   delay 50 s 	16.7772 MHz
; *******************************************************************************
_Delay_K:
	   push     03h	
	   nop				
	   mov	    R3,#30d		
	   djnz     R3,$

	   pop	    03h 		

	   ret	

; *******************************************************************************
; *   delay 4.069 ms 
; *******************************************************************************
_Delay_4ms:
	   push     03h		
	   mov	    R3,#79d

_L4:	   CALL	    _Delay_K		
	   djnz     R3,_L4

	   pop	    03h 		
	   ret	
			
			
; *******************************************************************************
; *   delay 10.04 ms 
; *******************************************************************************
_10ms:
	   push     03h 		
	   mov	    R3,#195d
		
_L10:	   call     _Delay_K		
	   djnz     R3,_L10

	   pop	    03h 		
	   ret	

; *******************************************************************************
; *   delay 21 ms 
; *******************************************************************************
_21ms:
	   push     03h
 	   nop	
	   mov	    R3,#75d
		
_L21:	   push	    03h
	   mov	    R3,#200d	
	   djnz     R3,$
	   pop	    03h
	   
	   djnz	    R3,_L21
	  	

	   pop	    03h 		
	   ret	

; *******************************************************************************
; *    delay 199 ms 
; *******************************************************************************
_200ms:
	   push     03h 		
	   mov	    R3,#24d	; 49d for 16,772 MHz	

_L3:	   call     _Delay_4ms		
	   djnz     R3,_L3

	   pop	    03h 		
	   ret

; *******************************************************************************
; *     send byte via COM
; *	byte in A
; *******************************************************************************
SENDCHAR:       
        JNB     TI,$            ; wait til present char gone
        CLR     TI              ; must clear TI
        MOV     SBUF,A
        RET

; *******************************************************************************
; *     converts low nibble A in char
; *	byte in A
; *******************************************************************************
HEX2ASCII:      
        ANL     A,#00Fh
        CJNE    A,#00Ah,$+3
        JC      IO0030
        ADD     A,#007h
IO0030: ADD     A,#'0'
        RET
; *******************************************************************************
; *	Send Debug Info via RS232
; *******************************************************************************
_Debug:
$IF (DEBUG = 1)
	push	PSW
	clr	C

	mov	DPTR,#DT13
	CALL	_STRING_COM
	mov	A,pBytesSend	; sended bytes
	CALL	_HEX2_COM
	CALL	_RET_COM

;	mov	R1,#pDebug
	mov	A,R1
	jz	_NDB
	mov	DPTR,#DT2
	CALL	_STRING_COM
	mov	A,@R1		; DataLength
	inc	R1
	dec	pBytesSend
	CALL	_HEX2_COM
	CALL	_RET_COM

	mov	A,R1
	jz	_NDB
	mov	DPTR,#DT3
	CALL	_STRING_COM
	mov	A,@R1		; Command
	inc	R1
	dec	pBytesSend
	CALL	SENDCHAR
	CALL	_RET_COM
	
	jnb	bFrame,_DLF	; only if data valid	
	mov	DPTR,#DT4
	CALL	_STRING_COM
	
	mov	R4,pBytesSend	; rest data
_DAL:	mov	A,@R1		; Bytes send from DataFrame
	CALL	_HEX2_COM
	inc	R1
	djnz	R4,_DAL
	CALL	_RET_COM

_DLF:	pop	PSW
	
	ret

_NDB:	mov	DPTR,#DT12
	CALL	_STRING_COM
	CALL	_RET_COM

	pop	PSW
$ENDIF	
	ret


DT1:	db	'No ACK for slave address',0h
DT2:	db	'Frame-Length from PC: ',0h
DT3:	db	'Command: ',0h
DT4:	db	'Frame: ',0h
DT5:	db	'Error at: ',0h
DT6:	db	'No connect to I2C ARM',0h
DT7:	db	'No ACK for 0x08',0h
DT8:	db	'No ACK for Read Request String',0h
DT9:	db	'TimeOut waiting Data USB',0h
DT10:	db	'Leave Ini',0h
DT11:	db	'TimeOut USB Input Frame',0h
DT12:	db	'No more data in debug buffer',0h
DT13:	db	'Bytes to I2C sended :',0h
DT14:	db	'Stopp by:',0h
DT15:	db	'TimeOut at:',0h
DT16:	db	'NAck at:',0h


B10: 	db	'0x08 senden, 24 Byte empfangen',0h
B11: 	db	'Run senden, 1 Byte empfangen',0h
B22: 	db	'2 Bytes senden, 2 Bytes empfangen',0h
DTE:	db	'data transfer error: ',0h
ID_STR:	db	'ADuC7026  D62  D3Y    ',0Ah,0Dh,00h

org 60h	
DongId:	db	'I2C Rev: 1.2   SW-Rev: 1.0     ',0h
org 80h
UserId:	db	'Bronkhorst High-Tech BV        ',0h

END
